﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.IO;

namespace Persistence {

  public class Root {

    [Storable]
    public int[] i = new int[] { 3, 4, 5, 6 };
    [Storable]
    public string s;
    [Storable]
    public ArrayList intArray = new ArrayList(new int[] { 1, 2, 3 });
    [Storable]
    public List<int> intList = new List<int>(new int[] { 321, 312, 321 });
    [Storable]
    public Custom c;
    [Storable]
    public List<Root> selfReferences;
    [Storable]
    public double[,] multiDimArray = new double[,] { { 1, 2, 3 }, { 3, 4, 5 } };
    [Storable]
    public bool boolean = true;
    [Storable]
    public DateTime dateTime = new DateTime();
    [Storable]
    public KeyValuePair<string, int> kvp = new KeyValuePair<string, int>("test key", 123);
    [Storable]
    public Dictionary<string, int> dict = new Dictionary<string, int>();
  }

  public class Custom {
    [Storable]
    public int i;
    [Storable]
    public Root r;
    [Storable]
    public string name = "Serial";
  }

  public class NewSerializationTest {
    public static void Main() {
      Root r = new Root();
      r.selfReferences = new List<Root>();
      r.selfReferences.Add(r);
      r.selfReferences.Add(r);
      r.c = new Custom();
      r.c.r = r;
      r.dict.Add("one", 1);
      r.dict.Add("two", 2);
      r.dict.Add("three", 3);

      Serializer s = new Serializer(r,
        new IPrimitiveSerializer[] {
          new String2XMLSerializer(),
          new Int2XMLSerializer(),
          new Double2XmlSerializer(),
          new Boolean2XmlSerializer(),
          new DateTime2XmlSerializer(),
        }, new ICompoundSerializer[] {
          new EnumerableSerializer(),
          new ArraySerializer(),
          new KeyValuePair2XmlSerializer(),
          new Dictionary2XmlSerializer(),
        });      
      Persistence.XmlFormatter xmlFormatter = new Persistence.XmlFormatter();
      StreamWriter writer = new StreamWriter("test.xml");
      foreach (ISerializationToken token in s) {
        string line = xmlFormatter.Format(token);
        writer.Write(line);
        Console.Out.Write(line);
      }
      writer.Close();
      XmlParser parser = new XmlParser(new StreamReader("test.xml"));
      DeSerializer deSerializer = new DeSerializer(
        new IPrimitiveSerializer[] { 
          new Int2XMLSerializer(),
          new String2XMLSerializer(),
          new Double2XmlSerializer(),
          new Boolean2XmlSerializer(),
          new DateTime2XmlSerializer(),
        },
        new ICompoundSerializer[] { 
          new ArraySerializer(),
          new EnumerableSerializer(),
          new KeyValuePair2XmlSerializer(),
          new Dictionary2XmlSerializer(),
        });      
      object o = deSerializer.DeSerialize(parser);
      Console.Out.WriteLine(Util.AutoFormat(o, true));
      Console.In.ReadLine();
    }
  }
}
